package webapp

import (
	"crypto/rsa"
	"fmt"
	"io/ioutil"

	"git.loafle.net/commons_go/logging"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	"git.loafle.net/overflow/overflow_gateway_websocket/rpc"
	jwt "github.com/dgrijalva/jwt-go"
	"github.com/valyala/fasthttp"
)

var (
	verifyKey *rsa.PublicKey
	signKey   *rsa.PrivateKey
)

func init() {
	signBytes, err := ioutil.ReadFile("overFlow-private.key")
	if nil != err {
		panic(err)
	}

	signKey, err = jwt.ParseRSAPrivateKeyFromPEM(signBytes)
	if nil != err {
		panic(err)
	}

	verifyBytes, err := ioutil.ReadFile("overFlow-public.pem")
	if nil != err {
		panic(err)
	}

	verifyKey, err = jwt.ParseRSAPublicKeyFromPEM(verifyBytes)
	if nil != err {
		panic(err)
	}
}

func newGatewayRPCHandler() rpc.GatewayRPCHandler {
	gh := &GatewayRPCHandlers{}

	return gh
}

type GatewayRPCHandlers struct {
	rpc.GatewayRPCHandlers
}

func (sh *GatewayRPCHandlers) Init(serverCTX cwf.ServerContext) error {
	return nil
}

func (sh *GatewayRPCHandlers) Handshake(socketCTX cwf.SocketContext, ctx *fasthttp.RequestCtx) (id string, extensionsHeader *fasthttp.ResponseHeader) {
	var ok bool
	// tokenString := string(ctx.Request.Header.Cookie("AuthToken"))
	tokenString := string(ctx.QueryArgs().Peek("AuthToken"))
	// tokenString := "eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.eyJhdWQiOiJ3d3cub3ZlcmZsb3cuY2xvdWQiLCJleHAiOjE1MjA4NTUwMDksImlhdCI6MTUyMDc2ODYwOSwiaXNzIjoib3ZlckZsb3ciLCJzdWIiOiJvdmVyZmxvd0Bsb2FmbGUuY29tIn0.YvpylMqHdL72p4Otm3y5L9-2zgvXvLFvt9MKK6-wF39_-EJLL-GgTiPVHx2--jr5ApuRIkJVRURVt7RtnGW65Te1to8u5Be6DnXenI5hRfGqfNLfoH0z_iNNbdjTDHLXa5TRE4Hf4oadYDgb7AotYi9-Mz0iNMyVNqPA4mzPO1ABvUPaHi89CwHYN2ZroaicQwaGxgWUX-KIjYXdxmWHi2shTJGpnHQd5RWeRN3H-AZVXkq05f4NSh6kpGIDu-e7lLHHSB4slWWQ0ML00c0C5fdnBJG4Xeisu195Y69ePZu_5lRpv2T3eH9mbEzUw_TEKZP-WGK_koDUGirYKFBaBJvJlaVOM3hUD3hYQ0DueNHqAwaDpr0dGuvSQwjr6tAqGnXscH6Gfj4x_hNMySVzMR-nWYeG7cDSNq9X5N-A93FKF_n-HCf11doDMsgM9rlpH04eVFCpZqNToLgsJVtEOUezuHSwOmnFTbLelquqUD02S_gGlowtE2CVBOccsfHZK9oTw1lInJYRjpxyevdWx1bvyMyZrf1Dk_PRsTtdboqd-RYp6zP1E4x3dky_VSe7AjUQI0t_SQpxL4ysmzs_hXhdiaQ9PwoHKkZW5Cz9z3oPuUi0iEatMx_9gvFUfVqaVRjNCW79ULGB900TixOxHy1UlNDFN9xip2Mmr7qZwrI"
	token, err := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		if _, ok := token.Method.(*jwt.SigningMethodRSA); !ok {
			return nil, fmt.Errorf("Webapp: Unexpected signing method: %v", token.Header["alg"])
		}

		// hmacSampleSecret is a []byte containing your secret, e.g. []byte("my_secret_key")
		return verifyKey, nil
	})

	if nil != err {
		return "", nil
	}

	var claims jwt.MapClaims
	if claims, ok = token.Claims.(jwt.MapClaims); !ok || !token.Valid {
		logging.Logger().Warn(fmt.Sprintf("Webapp: Token is not valid %v", token))
		return "", nil
	}

	return claims["sub"].(string), nil
}

func (sh *GatewayRPCHandlers) Validate() {
	sh.GatewayRPCHandlers.Validate()
}
